/**
 * This file is part of pppBenchmark, a program accompanying
 * the paper: Abeel, T, Van de Peer Y, Saeys Y.  2009.  Towards a gold standard for promoter prediction evaluation. Bioinformatics. 25(12):i313-i320.
    * 
 * Copyright 2009 Thomas Abeel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *
 */
package be.abeel.pppbenchmark;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import be.abeel.io.LineIterator;

/**
 * protocol based on the work by Abeel et al.
 * 
 * @author Thomas Abeel
 * 
 */
public class PredTagsProtocol extends Protocol {

	
	public Map<Double, int[]> run(String chr, LineIterator pred) throws Exception {
		System.out.println("\tRunning PredTags: "+chr);
		Map<Double, int[]> out = new HashMap<Double, int[]>();
//		System.out.println("# chromosome=" + chr);
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(5);

		ArrayList<Tag> tags = new ArrayList<Tag>();
		ArrayList<Prediction> predictions = new ArrayList<Prediction>();

		/* Load tags for the selected chromosome */
		LineIterator it = new LineIterator(Config.cage());
		for (String line : it) {
			String[] arr = line.split("\t");
			if (arr[0].equalsIgnoreCase(chr)) {
				int start = Integer.parseInt(arr[3]);
				int end = Integer.parseInt(arr[4]);
				tags.add(new Tag(start - (int) Config.distance, end + (int) Config.distance));
			}

		}
		it.close();
		/* Load predictions for the selected chromosome */

		for (String line : pred) {
			// System.err.println(line);
			String[] arr = line.split("\t");
			if (arr[0].equalsIgnoreCase(chr)) {
				int start = Integer.parseInt(arr[1]);
				int end = Integer.parseInt(arr[2]);
				double score = Double.parseDouble(arr[3]);
				// System.out.println(score);
				predictions.add(new Prediction(start, end, score));

			}

		}
		pred.close();

		// int total = 0;
		// int overlap = 0;
		//
		// for (Prediction p : predictions) {
		// total++;
		// if (p.overlap)
		// overlap++;
		// }
		// System.out.println("# Total=" + total);
		// System.out.println("# Overlap=" + overlap);
//		System.out.println("# Distinct tags set = " + tags.size());
//		System.out.println("# Predictions = " + predictions.size());
		// System.out.println("# minimum score = " + min);
		// System.out.println("# maximum score = " + max);
		/* Do analysis with moving threshold */
		for (Tag t : tags) {
			for (Prediction p : predictions) {
				if (p.overlaps(t)) {
					t.setHighest(p.score);
					p.overlap = true;
				}
			}
		}

		double range = 1 / Config.steps;
		if (range < 0.00000001) {
			System.err.println("PredTags: Modified range!");
			range++;

		}

		// for (Tag t : tags) {
		// System.out.println("# Tagscore: " + t.lowestAssociated);
		// }
//		System.out.println("#\tpred (corr/tot)\ttags (corr/tot)\tprec.\trecall");
		for (double d = 0; d <= 1; d += range) {
			int predictedTags = 0;
			int totalTags = 0;
			int correctPredictions = 0;
			int totalPredictions = 0;

			// does not work?
			for (Tag t : tags) {
				totalTags++;
				if (t.highestScore >= d)
					predictedTags++;
			}

			// works?
			for (Prediction p : predictions) {
				if (p.score >= d && p.overlap)
					correctPredictions++;
				if (p.score >= d)
					totalPredictions++;

			}
//			System.out.println(nf.format(d) + "\t" + correctPredictions + "\t" + totalPredictions + "\t" + predictedTags + "\t" + totalTags + "\t" + nf.format((double) correctPredictions / totalPredictions) + "\t"
//					+ nf.format((double) predictedTags / totalTags));
			out.put(d, new int[] { correctPredictions, totalPredictions, predictedTags, totalTags });

		}
		return out;

	}
}
